import { MonadPlus } from '../monadplus';
export declare class Sequence<a, z> extends MonadPlus<a> implements Iterable<a> {
    protected readonly cons: (z: z, cons: (a?: a, z?: z) => Sequence.Data<a, z>) => Sequence.Data<a, z>;
    constructor(cons: (z: z, cons: (a?: a, z?: z) => Sequence.Data<a, z>) => Sequence.Data<a, z>);
}
export declare namespace Sequence {
    function resume<a>(iterator: Sequence.Iterator<a>): Sequence<a, Sequence.Iterator<a>>;
    function from<a>(as: Iterable<a>): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
    function cycle<a>(as: Iterable<a>): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
    function random(): Sequence<number, [number, Map<number, Sequence.Thunk<number>>]>;
    function random<a>(gen: () => a): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
    function random<a>(as: a[]): Sequence<a, Sequence.Iterator<number>>;
    function concat<a>(as: Sequence<Sequence<a, unknown>, unknown>): Sequence<a, [Sequence.Iterator<Sequence<a, unknown>>, Sequence.Iterator<a>]>;
    function zip<a, b>(a: Sequence<a, unknown>, b: Sequence<b, unknown>): Sequence<[a, b], [Sequence.Iterator<a>, Sequence.Iterator<b>]>;
    function difference<a>(a: Sequence<a, unknown>, b: Sequence<a, unknown>, cmp: (l: a, r: a) => number): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
    function union<a>(a: Sequence<a, unknown>, b: Sequence<a, unknown>, cmp: (l: a, r: a) => number): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
    function intersect<a>(a: Sequence<a, unknown>, b: Sequence<a, unknown>, cmp: (l: a, r: a) => number): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
    function fmap<a, b>(m: Sequence<a, unknown>, f: (a: a) => b): Sequence<b, Sequence.Iterator<a>>;
    function fmap<a>(m: Sequence<a, unknown>): <b>(f: (a: a) => b) => Sequence<b, Sequence.Iterator<a>>;
    function pure<a>(a: a): Sequence<a, number>;
    function ap<a, b>(mf: Sequence<(a: a) => b, unknown>, ma: Sequence<a, unknown>): Sequence<b, [Sequence.Iterator<Sequence<b, unknown>>, Sequence.Iterator<b>]>;
    function ap<a, b>(mf: Sequence<(a: a) => b, unknown>): (ma: Sequence<a, unknown>) => Sequence<b, [Sequence.Iterator<Sequence<b, unknown>>, Sequence.Iterator<b>]>;
    const Return: typeof pure;
    function bind<a, b>(m: Sequence<a, unknown>, f: (a: a) => Sequence<b, unknown>): Sequence<b, Sequence.Iterator<a>>;
    function bind<a>(m: Sequence<a, unknown>): <b>(f: (a: a) => Sequence<b, unknown>) => Sequence<b, Sequence.Iterator<a>>;
    function sequence<b>(ms: Sequence<b, unknown>[]): Sequence<Sequence<b, [Sequence.Iterator<b>, Sequence.Iterator<b>]>, Sequence.Iterator<Sequence<b, [Sequence.Iterator<b>, Sequence.Iterator<b>]>>>;
    const mempty: Sequence<never, never>;
    function mappend<a>(l: Sequence<a, unknown>, r: Sequence<a, unknown>): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
    function mconcat<a>(as: Iterable<Sequence<a, unknown>>): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
    const mzero: Sequence<never, never>;
    function mplus<a>(l: Sequence<a, unknown>, r: Sequence<a, unknown>): Sequence<a, [Sequence.Iterator<a>, Sequence.Iterator<a>]>;
}
export interface Sequence<a, z> {
    extract(): a[];
    [Symbol.iterator](): Iterator<a>;
    iterate(): Sequence.Thunk<a>;
    fmap<b>(f: (a: a) => b): Sequence<b, Sequence.Iterator<a>>;
    ap<a, z>(this: Sequence<(a: a) => z, unknown>, a: Sequence<a, unknown>): Sequence<z, [Sequence.Iterator<Sequence<z, unknown>>, Sequence.Iterator<z>]>;
    ap<a, b, z>(this: Sequence<(a: a, b: b) => z, unknown>, a: Sequence<a, unknown>): Sequence<(b: b) => z, [Sequence.Iterator<Sequence<z, unknown>>, Sequence.Iterator<z>]>;
    ap<a, b, c, z>(this: Sequence<(a: a, b: b, c: c) => z, unknown>, a: Sequence<a, unknown>): Sequence<(b: b, c: c) => z, [Sequence.Iterator<Sequence<z, unknown>>, Sequence.Iterator<z>]>;
    ap<a, b, c, d, z>(this: Sequence<(a: a, b: b, c: c, d: d) => z, unknown>, a: Sequence<a, unknown>): Sequence<(b: b, c: c, d: d) => z, [Sequence.Iterator<Sequence<z, unknown>>, Sequence.Iterator<z>]>;
    ap<a, b, c, d, e, z>(this: Sequence<(a: a, b: b, c: c, d: d, e: e) => z, unknown>, a: Sequence<a, unknown>): Sequence<(b: b, c: c, d: d, e: e) => z, [Sequence.Iterator<Sequence<z, unknown>>, Sequence.Iterator<z>]>;
    bind<b>(f: (a: a) => Sequence<b, unknown>): Sequence<b, [Sequence.Iterator<Sequence<b, unknown>>, Sequence.Iterator<b>]>;
    join<b>(this: Sequence<Sequence<b, unknown>, unknown>): Sequence<b, [Sequence.Iterator<Sequence<b, unknown>>, Sequence.Iterator<b>]>;
    mapM<b>(f: (a: a) => Sequence<b, unknown>): Sequence<b[], [Sequence.Iterator<Sequence<b[], unknown>>, Sequence.Iterator<b[]>]>;
    filterM(f: (a: a) => Sequence<boolean, unknown>): Sequence<a[], [Sequence.Iterator<Sequence<a[], unknown>>, Sequence.Iterator<a[]>]>;
    map<b>(f: (a: a, i: number) => b): Sequence<b, Sequence.Iterator<a>>;
    filter(f: (a: a, i: number) => boolean): Sequence<a, Sequence.Iterator<a>>;
    scanl<b>(f: (b: b, a: a) => b, z: b): Sequence<b, [b, Sequence.Iterator<a>, number]>;
    foldr<b>(f: (a: a, b: Sequence<b, unknown>) => Sequence<b, unknown>, z: Sequence<b, unknown>): Sequence<b, [Sequence.Iterator<Sequence<b, unknown>>, Sequence.Iterator<b>]>;
    group(f: (x: a, y: a) => boolean): Sequence<a[], [Sequence.Iterator<a>, a[]]>;
    inits(): Sequence<a[], [Sequence.Iterator<a[]>, Sequence.Iterator<a[]>]>;
    tails(): Sequence<a[], [Sequence.Iterator<a[]>, Sequence.Iterator<a[]>]>;
    segs(): Sequence<a[], [Sequence.Iterator<a[]>, Sequence.Iterator<a[]>]>;
    subsequences(): Sequence<a[], [Sequence.Iterator<a[]>, Sequence.Iterator<a[]>]>;
    permutations(): Sequence<a[], [Sequence.Iterator<Sequence<a[], unknown>>, Sequence.Iterator<a[]>]>;
    take(n: number): Sequence<a, Sequence.Iterator<a>>;
    drop(n: number): Sequence<a, Sequence.Iterator<a>>;
    takeWhile(f: (a: a) => boolean): Sequence<a, Sequence.Iterator<a>>;
    dropWhile(f: (a: a) => boolean): Sequence<a, Sequence.Iterator<a>>;
    takeUntil(f: (a: a) => boolean): Sequence<a, Sequence.Iterator<a>>;
    dropUntil(f: (a: a) => boolean): Sequence<a, Sequence.Iterator<a>>;
    sort(cmp?: (a: a, b: a) => number): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
    unique(): Sequence<a, Sequence.Iterator<a>>;
    memoize(): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
    reduce(): Sequence<a, [number, Map<number, Sequence.Thunk<a>>]>;
}
export declare namespace Sequence {
    type Data<a, z> = [] | [a] | [a, z];
    namespace Data {
        function cons<a, z>(): [];
        function cons<a, z>(a: a): [a];
        function cons<a, z>(a: a, z: z): [a, z];
    }
    type Thunk<a> = [a, Iterator<a>, number];
    namespace Thunk {
        function value<a>(thunk: Thunk<a>): a;
        function iterator<a>(thunk: Thunk<a>): Iterator<a>;
        function index<a>(thunk: Thunk<a>): number;
    }
    type Iterator<a> = () => Thunk<a>;
    namespace Iterator {
        const done: Sequence.Iterator<never>;
        function when<a, b>(thunk: Thunk<a>, caseDone: (thunk: Thunk<a>) => b, caseIterable: (thunk: Thunk<a>, recur: () => b) => b): b;
    }
    function isIterable(thunk: Thunk<unknown>): boolean;
    namespace Exception {
        function invalidConsError(args: IArguments): TypeError;
        function invalidDataError(data: unknown[]): TypeError;
        function invalidThunkError(thunk: Sequence.Thunk<unknown>): TypeError;
    }
}
